@@ -68,7 +68,7 @@ class Sql_UserInfo:
 
     @staticmethod
     def _sql_batch_select_data(qqs: list):
-        return f"select * from `info` where `qq` in {tuple(qqs)};"
+        return f"select * from `info` where `qq` in {Sql.utils.tupleify(qqs)};"
 
     @staticmethod
     def _sql_delete_single_data(qq: int):
@@ -82,6 +82,14 @@ class Sql_UserInfo:
             "latest_speech_group": data[2],
         }
 
+    @classmethod
+    def select_single_data(cls, qq: int):
+        sql_ins.cursor.execute(cls._sql_select_single_data(qq))
+        one = sql_ins.cursor.fetchone()
+        if one is None:
+            return None
+        return cls.deserialize(one)
+
     @classmethod
     def select_batch_data_by_qqs(cls, qqs: list):
         sql_ins.cursor.execute(cls._sql_batch_select_data(qqs))
@@ -108,7 +116,7 @@ class Sql_rebirth:
 
     @staticmethod
     def _sql_batch_select_data(qqs: list):
-        return f"select * from `rebirth` where `qq` in {tuple(qqs)};"
+        return f"select * from `rebirth` where `qq` in {Sql.utils.tupleify(qqs)};"
 
     @staticmethod
     def _sql_check_table_exists():
@@ -188,7 +196,7 @@ class Sql_badge:
 
     @staticmethod
     def _sql_batch_select_data(qqs: list):
-        return f"select * from `badge` where `qq` in {tuple(qqs)};"
+        return f"select * from `badge` where `qq` in {Sql.utils.tupleify(qqs)};"
 
     @staticmethod
     def _sql_update_single_data(data: dict):
@@ -256,30 +264,33 @@ class Sql_badge:
 
 class DB_Badge:
     @staticmethod
-    def init_user_data(qq: int):
-        data = Sql_badge.select_single_data(qq)
-        if data is None:
-            data = {
-                "qq": qq,
-                "badge_ids": "",
-                "glue_me_count": 0,
-                "glue_target_count": 0,
-                "glue_plus_count": 0,
-                "glue_plus_length_total": 0,
-                "glue_punish_count": 0,
-                "glue_punish_length_total": 0,
-                "pk_win_count": 0,
-                "pk_lose_count": 0,
-                "pk_plus_length_total": 0,
-                "pk_punish_length_total": 0,
-                "lock_me_count": 0,
-                "lock_target_count": 0,
-                "lock_plus_count": 0,
-                "lock_punish_count": 0,
-                "lock_plus_length_total": 0,
-                "lock_punish_length_total": 0,
-            }
-            Sql_badge.insert_single_data(data)
+    def init_user_data(qq: int, at_qq: int = None):
+        for account in [qq, at_qq]:
+            if account is None:
+                continue
+            data = Sql_badge.select_single_data(account)
+            if data is None:
+                data = {
+                    "qq": account,
+                    "badge_ids": "",
+                    "glue_me_count": 0,
+                    "glue_target_count": 0,
+                    "glue_plus_count": 0,
+                    "glue_plus_length_total": 0,
+                    "glue_punish_count": 0,
+                    "glue_punish_length_total": 0,
+                    "pk_win_count": 0,
+                    "pk_lose_count": 0,
+                    "pk_plus_length_total": 0,
+                    "pk_punish_length_total": 0,
+                    "lock_me_count": 0,
+                    "lock_target_count": 0,
+                    "lock_plus_count": 0,
+                    "lock_punish_count": 0,
+                    "lock_plus_length_total": 0,
+                    "lock_punish_length_total": 0,
+                }
+                Sql_badge.insert_single_data(data)
 
     @classmethod
     def plus_value_by_ley(cls, qq: int, key: str, plus_value: int):
@@ -385,7 +396,7 @@ class Sql_farm:
 
     @staticmethod
     def _sql_batch_select_data(qqs: list):
-        return f"select * from `farm` where `qq` in {tuple(qqs)};"
+        return f"select * from `farm` where `qq` in {Sql.utils.tupleify(qqs)};"
 
     @staticmethod
     def _sql_update_single_data():
@@ -441,19 +452,22 @@ class Sql_farm:
 
 class DB_Farm:
     @staticmethod
-    def init_user_data(qq: int):
-        data = Sql_farm.select_signle_data(qq)
-        if data is None:
-            Sql_farm.insert_single_data(
-                {
-                    "qq": qq,
-                    "farm_status": FarmConst.status_empty,
-                    "farm_latest_plant_time": TimeConst.DEFAULT_NONE_TIME,
-                    "farm_need_time": 0,
-                    "farm_count": 0,
-                    "farm_expect_get_length": 0,
-                }
-            )
+    def init_user_data(qq: int, at_qq: int = None):
+        for account in [qq, at_qq]:
+            if account is None:
+                continue
+            data = Sql_farm.select_signle_data(account)
+            if data is None:
+                Sql_farm.insert_single_data(
+                    {
+                        "qq": account,
+                        "farm_status": FarmConst.status_empty,
+                        "farm_latest_plant_time": TimeConst.DEFAULT_NONE_TIME,
+                        "farm_need_time": 0,
+                        "farm_count": 0,
+                        "farm_expect_get_length": 0,
+                    }
+                )
 
     @staticmethod
     def get_user_data(qq: int):
@@ -464,12 +478,131 @@ class DB_Farm:
         Sql_farm.update_single_data(data)
 
 
+class Sql_friends:
+    @staticmethod
+    def _sql_create_table():
+        return "create table if not exists `friends` (`qq` bigint, `friends_list` varchar(255), `friends_share_count` integer, `friends_cost_latest_time` varchar(255), `friends_will_collect_length` float, `friends_collect_latest_time` varchar(255), primary key (`qq`));"
+
+    @staticmethod
+    def _sql_insert_single_data():
+        return "insert into `friends` (`qq`, `friends_list`, `friends_share_count`, `friends_cost_latest_time`, `friends_will_collect_length`, `friends_collect_latest_time`) values (:qq, :friends_list, :friends_share_count, :friends_cost_latest_time, :friends_will_collect_length, :friends_collect_latest_time);"
+
+    @staticmethod
+    def _sql_select_single_data(qq: int):
+        return f"select * from `friends` where `qq` = {qq};"
+
+    @staticmethod
+    def _sql_batch_select_data(qqs: list):
+        return f"select * from `friends` where `qq` in {Sql.utils.tupleify(qqs)};"
+
+    @staticmethod
+    def _sql_update_single_data():
+        return "update `friends` set `friends_list` = :friends_list, `friends_share_count` = :friends_share_count, `friends_cost_latest_time` = :friends_cost_latest_time, `friends_will_collect_length` = :friends_will_collect_length, `friends_collect_latest_time` = :friends_collect_latest_time where `qq` = :qq;"
+
+    @staticmethod
+    def _sql_delete_single_data(qq: int):
+        return f"delete from `friends` where `qq` = {qq};"
+
+    @staticmethod
+    def _sql_check_table_exists():
+        return (
+            'select count(*) from sqlite_master where type="table" and name="friends";'
+        )
+
+    @staticmethod
+    def deserialize(data: tuple):
+        return {
+            "qq": data[0],
+            "friends_list": data[1],
+            "friends_share_count": data[2],
+            "friends_cost_latest_time": data[3],
+            "friends_will_collect_length": data[4],
+            "friends_collect_latest_time": data[5],
+        }
+
+    @classmethod
+    def select_signle_data(cls, qq: int):
+        sql_ins.cursor.execute(cls._sql_select_single_data(qq))
+        one = sql_ins.cursor.fetchone()
+        if one is None:
+            return None
+        return cls.deserialize(one)
+
+    @classmethod
+    def insert_single_data(cls, data: dict):
+        sql_ins.cursor.execute(cls._sql_insert_single_data(), data)
+        sql_ins.conn.commit()
+
+    @classmethod
+    def update_single_data(cls, data: dict):
+        sql_ins.cursor.execute(cls._sql_update_single_data(), data)
+        sql_ins.conn.commit()
+
+    @classmethod
+    def delete_single_data(cls, qq: int):
+        sql_ins.cursor.execute(cls._sql_delete_single_data(qq))
+        sql_ins.conn.commit()
+
+    @classmethod
+    def select_batch_data_by_qqs(cls, qqs: list):
+        sql_ins.cursor.execute(cls._sql_batch_select_data(qqs))
+        return [cls.deserialize(data) for data in sql_ins.cursor.fetchall()]
+
+
+class DB_Friends:
+    @staticmethod
+    def init_user_data(qq: int, at_qq: int = None):
+        for account in [qq, at_qq]:
+            if account is None:
+                continue
+            data = Sql_friends.select_signle_data(account)
+            if data is None:
+                Sql_friends.insert_single_data(
+                    {
+                        "qq": account,
+                        "friends_list": "",
+                        "friends_share_count": 0,
+                        "friends_cost_latest_time": TimeConst.DEFAULT_NONE_TIME,
+                        "friends_will_collect_length": 0,
+                        "friends_collect_latest_time": TimeConst.DEFAULT_NONE_TIME,
+                    }
+                )
+
+    @staticmethod
+    def get_user_data(qq: int):
+        return Sql_friends.select_signle_data(qq)
+
+    @staticmethod
+    def update_user_data(data: dict):
+        # FIXME: 兜底数据格式
+        is_friends_list_typeof_arr = isinstance(data["friends_list"], list)
+        if is_friends_list_typeof_arr:
+            # stringify and join
+            new_list = ",".join([str(q) for q in data["friends_list"]])
+            data["friends_list"] = new_list
+        Sql_friends.update_single_data(data)
+
+    @staticmethod
+    def get_batch_user_data(qqs: list):
+        return Sql_friends.select_batch_data_by_qqs(qqs)
+
+class Sql_utils():
+
+    @staticmethod
+    def tupleify(data: list):
+        if len(data) == 1:
+            return f'({data[0]})'
+        return tuple(data)
+
 class Sql:
 
     sub_table_info = Sql_UserInfo()
     sub_table_rebirth = Sql_rebirth()
     sub_table_badge = Sql_badge()
     sub_table_farm = Sql_farm()
+    sub_table_friends = Sql_friends()
+
+    utils = Sql_utils()
 
     def __init__(self):
         self.sqlite_path = Paths.sqlite_path()
@@ -505,6 +638,10 @@ class Sql:
         max = Config.get_config("ranking_list_length")
         return f"select * from `users` order by `length` desc limit {max};"
 
+    @staticmethod
+    def __sql_select_batch_data(qqs: list):
+        return f"select * from `users` where `qq` in {Sql.utils.tupleify(qqs)};"
+
     @classmethod
     def get_top_users(cls) -> list:
         sql_ins.cursor.execute(cls.__sql_order_by_length())
@@ -551,6 +688,12 @@ class Sql:
             return None
         return cls.deserialize(one)
 
+    @classmethod
+    def select_batch_data(cls, qqs: list):
+        sql_ins.cursor.execute(cls.__sql_select_batch_data(qqs))
+        some = sql_ins.cursor.fetchall()
+        return [cls.deserialize(one) for one in some]
+
     @classmethod
     def check_table_exists(cls):
         create_table_funs = [
@@ -571,6 +714,10 @@ class Sql:
                 cls.sub_table_farm._sql_check_table_exists,
                 cls.sub_table_farm._sql_create_table,
             ],
+            [
+                cls.sub_table_friends._sql_check_table_exists,
+                cls.sub_table_friends._sql_create_table,
+            ],
         ]
         # check users, info, rebirth table exists
         for funs in create_table_funs:
@@ -623,6 +770,14 @@ class DB_UserInfo:
             )
         sql_ins.conn.commit()
 
+    @staticmethod
+    def get_batch_user_infos(qqs: list):
+        return Sql.sub_table_info.select_batch_data_by_qqs(qqs)
+
+    @staticmethod
+    def get_user_info(qq: int):
+        return Sql.sub_table_info.select_single_data(qq)
+
 
 class DataUtils:
     @staticmethod
@@ -634,13 +789,16 @@ class DataUtils:
         return {one["qq"]: one for one in data}
 
     @classmethod
-    def merge_data(cls, data_1: dict, data_2: dict):
-        # handle None
-        if data_1 is None:
-            data_1 = {}
-        if data_2 is None:
-            data_2 = {}
-        return cls.__assign(data_1, data_2)
+    def merge_data(cls, *datas: list):
+        first_data = datas[0]
+        if first_data is None:
+            first_data = {}
+        for i in range(1, len(datas)):
+            value = datas[i]
+            if value is None:
+                continue
+            first_data = cls.__assign(first_data, value)
+        return first_data
 
     @classmethod
     def merge_data_list(cls, datas: list):
@@ -654,13 +812,14 @@ class DataUtils:
             result.append(maps[0][user["qq"]])
         return result
 
-
 class DB:
 
     sub_db_info = DB_UserInfo()
     sub_db_rebirth = DB_Rebirth()
     sub_db_badge = DB_Badge()
     sub_db_farm = DB_Farm()
+    sub_db_friends = DB_Friends()
+
     utils = DataUtils()
 
     @staticmethod
@@ -830,6 +989,10 @@ class DB:
         )
         return merged
 
+    @staticmethod
+    def get_batch_users(qqs: list):
+        return Sql.select_batch_data(qqs)
+
 
 def lazy_init_database():
     Sql.init_database()
