@@ -7,12 +7,13 @@ from .rebirth import RebirthSystem
 from .badge import BadgeSystem
 from .constants import OpFrom, TimeConst, FarmConst
 from .farm import FarmSystem
+from .friends import FriendsSystem
 from typing import Optional
 
 KEYWORDS = {
     "chinchin": ["牛子"],
     "pk": ["pk"],
-    "lock_me": ["🔒我", "suo我", "嗦我", "锁我"],
+    "lock_me": ["🔒我"],
     "lock": ["🔒", "suo", "嗦", "锁"],
     "glue": ["打胶"],
     "see_chinchin": ["看他牛子", "看看牛子"],
@@ -23,9 +24,34 @@ KEYWORDS = {
     # farm
     "farm": ["牛子仙境"],
     "farm_start": ["牛子修炼", "牛子练功", "牛子修仙"],
+    # friends
+    "friends": ["牛友", '牛子好友', '牛子朋友'],
+    "friends_add": ["关注牛子", "添加牛友", "添加朋友"],
+    "friends_delete": ["取关牛子", "删除牛友", "删除朋友"],
+    # help
+    "help": ["牛子帮助"],
 }
 
-DEFAULT_NONE_TIME = TimeConst.DEFAULT_NONE_TIME
+VERSION = '2.6.0'
+HELPPER = f"牛了个牛 v{VERSION}\n可用的指令/功能有：\n" + "、".join(
+    [
+        KEYWORDS.get("sign_up")[0],
+        KEYWORDS.get("chinchin")[0],
+        f"@某人 {KEYWORDS.get('see_chinchin')[0]}",
+        f"@某人 {KEYWORDS.get('pk')[0]}",
+        KEYWORDS.get("lock_me")[0],
+        f"@某人 {KEYWORDS.get('lock')[0]}",
+        KEYWORDS.get("glue")[0],
+        KEYWORDS.get("ranking")[0],
+        KEYWORDS.get("rebirth")[0],
+        KEYWORDS.get("badge")[0],
+        KEYWORDS.get("farm")[0],
+        KEYWORDS.get("farm_start")[0],
+        KEYWORDS.get("friends")[0],
+        f"@某人 {KEYWORDS.get('friends_add')[0]}",
+        f"@某人 {KEYWORDS.get('friends_delete')[0]}",
+    ]
+)
 
 
 def message_processor(
@@ -48,8 +74,9 @@ def message_processor(
     TODO：牛子成就额外的提示语
     TODO: 物品系统
     TODO: 抽取 utils 文件的导入
+    TODO: 牛子共享排行榜
 
-    TODO: 朋友系统
+    TODO：朋友加成
     """
     # lazy init database
     lazy_init_database()
@@ -65,9 +92,9 @@ def message_processor(
 
     # 消息上下文，用于追加消息
     msg_ctx = {"before": [get_at_segment(qq)], "after": []}
-    # hack send message impl
 
     def create_send_message_hook(origin_send_message):
+        # hack send message impl
         def send_message_hook(qq, group, message):
             before = join(msg_ctx["before"], "\n")
             content = None
@@ -98,9 +125,11 @@ def message_processor(
         },
     )
     # 初始化数据 - badge
-    DB.sub_db_badge.init_user_data(qq)
+    DB.sub_db_badge.init_user_data(qq, at_qq)
     # 初始化数据 - farm
-    DB.sub_db_farm.init_user_data(qq)
+    DB.sub_db_farm.init_user_data(qq, at_qq)
+    # 初始化数据 - friends
+    DB.sub_db_friends.init_user_data(qq, at_qq)
 
     # flow context
     ctx = {
@@ -110,6 +139,10 @@ def message_processor(
         "msg_ctx": msg_ctx,
     }
 
+    # 牛子帮助 (search)
+    if match_func(KEYWORDS.get("help"), message):
+        return Chinchin_help.entry_help(ctx)
+
     # 注册牛子
     if match_func(KEYWORDS.get("sign_up"), message):
         return Chinchin_me.sign_up(ctx)
@@ -133,6 +166,21 @@ def message_processor(
     if badge_msg:
         msg_ctx["before"].append(badge_msg)
 
+    # 检查朋友
+    friends_daily_info = FriendsSystem.check_friends_daily(qq)
+    if friends_daily_info:
+        msg_ctx["before"].append(friends_daily_info["message"])
+        friends_profit = friends_daily_info["profit"]
+        if friends_profit > 0:
+            DB.length_increase(
+                qq,
+                Chinchin_intercepor.length_operate(
+                    qq, friends_profit, source=OpFrom.FRIENDS_COLLECT, at_qq=at_qq
+                ),
+            )
+        else:
+            DB.length_decrease(qq, -friends_profit)
+
     # 检查修炼状态
     is_current_planting = Chinchin_farm.check_planting_status(ctx)
 
@@ -165,6 +213,10 @@ def message_processor(
         else:
             return Chinchin_upgrade.entry_rebirth(ctx)
 
+    # 牛友 (search)
+    if match_func(KEYWORDS.get("friends"), message):
+        return Chinchin_friends.entry_friends(ctx)
+    
     # 查询牛子信息 (search)
     # FIXME: 注意因为是模糊匹配，所以 “牛子” 的命令要放到所有 "牛子xxx" 命令的最后
     if match_func(KEYWORDS.get("chinchin"), message):
@@ -196,6 +248,15 @@ def message_processor(
         # 看别人的牛子
         if match_func(KEYWORDS.get("see_chinchin"), message):
             return Chinchin_info.entry_see_chinchin(ctx)
+
+        # 牛友交友
+        if match_func(KEYWORDS.get("friends_add"), message):
+            return Chinchin_friends.entry_friends_add(ctx)
+
+        # 牛友友尽
+        if match_func(KEYWORDS.get("friends_delete"), message):
+            return Chinchin_friends.entry_friends_delete(ctx)
+
     else:
         # 🔒自己
         if match_func(KEYWORDS.get("lock_me"), message):
@@ -208,13 +269,26 @@ def message_processor(
 
 class Chinchin_intercepor:
     @staticmethod
-    def length_operate(qq: int, origin_change: float, source: str = OpFrom.OTHER):
+    def length_operate(qq: int, origin_change: float, source: str = OpFrom.OTHER, at_qq: int = None):
+        # 转生加成
         rebirth_weight = RebirthSystem.get_weight_by_qq(qq)
         result = origin_change * rebirth_weight
+        # 成就加成
         result = BadgeSystem.handle_weighting_by_qq(qq, result, source)
+        # 朋友加成
+        result = FriendsSystem.handle_weighting(qq, at_qq=at_qq, length=result, source=source)
+        # fixed
         result = fixed_two_decimal_digits(result, to_number=True)
         return result
 
+    @staticmethod
+    def length_weight(qq: int, origin_length: float, source: str = OpFrom.OTHER, at_qq: int = None):
+        result = origin_length
+        # 朋友加成
+        result = FriendsSystem.handle_weighting(qq, at_qq=at_qq, length=result, source=source)
+        # fixed
+        result = fixed_two_decimal_digits(result, to_number=True)
+        return result
 
 class Chinchin_view:
     @staticmethod
@@ -271,10 +345,8 @@ class Chinchin_info:
                 prefix = "🥈"
             elif idx == 3:
                 prefix = "🥉"
-            if "latest_speech_nickname" not in user:
-                user["latest_speech_nickname"] = ""
-            nickname = user["latest_speech_nickname"]
-            if len(nickname) == 0:
+            nickname = user.get("latest_speech_nickname")
+            if not nickname:
                 nickname = "无名英雄"
             badge = BadgeSystem.get_first_badge_by_badge_string_arr(
                 user.get("badge_ids")
@@ -301,7 +373,8 @@ class Chinchin_info:
         qq = ctx["qq"]
         group = ctx["group"]
         at_qq = ctx["at_qq"]
-        target_chinchin_info = ChinchinInternal.internal_get_chinchin_info(at_qq)
+        target_chinchin_info = ChinchinInternal.internal_get_chinchin_info(
+            at_qq)
         msg_text = join(target_chinchin_info, "\n")
         msg_text = msg_text.replace("【牛子信息】", "【对方牛子信息】")
         send_message(qq, group, msg_text)
@@ -326,32 +399,48 @@ class ChinchinInternal:
         )
         # length
         message_arr.append(f"长度: {length_label}")
+        # friends
+        friends_info = FriendsSystem.get_friends_data(qq)
+        share_need_cost = friends_info['friends_need_cost']
+        if share_need_cost > 0:
+            share_text = None
+            share_count = friends_info['friends_share_count']
+            if share_count > 0:
+                share_text = f"{share_count}人共享"
+            message_arr.append(
+                join([
+                    f"好友费: {share_need_cost}cm",
+                    share_text
+                ], '，')
+            )
         # locked
-        if user_data.get("locked_time") != DEFAULT_NONE_TIME:
+        if user_data.get("locked_time") != TimeConst.DEFAULT_NONE_TIME:
             message_arr.append(
                 "最近被🔒时间: {}".format(
                     ArrowUtil.date_improve(user_data.get("locked_time"))
                 )
             )
         # pk
-        if user_data.get("pk_time") != DEFAULT_NONE_TIME:
+        if user_data.get("pk_time") != TimeConst.DEFAULT_NONE_TIME:
             message_arr.append(
-                "最近pk时间: {}".format(ArrowUtil.date_improve(user_data.get("pk_time")))
+                "最近pk时间: {}".format(
+                    ArrowUtil.date_improve(user_data.get("pk_time")))
             )
         # pked
-        if user_data.get("pked_time") != DEFAULT_NONE_TIME:
+        if user_data.get("pked_time") != TimeConst.DEFAULT_NONE_TIME:
             message_arr.append(
-                "最近被pk时间: {}".format(ArrowUtil.date_improve(user_data.get("pked_time")))
+                "最近被pk时间: {}".format(ArrowUtil.date_improve(
+                    user_data.get("pked_time")))
             )
         # glueing
-        if user_data.get("glueing_time") != DEFAULT_NONE_TIME:
+        if user_data.get("glueing_time") != TimeConst.DEFAULT_NONE_TIME:
             message_arr.append(
                 "最近打胶时间: {}".format(
                     ArrowUtil.date_improve(user_data.get("glueing_time"))
                 )
             )
         # glued
-        if user_data.get("glued_time") != DEFAULT_NONE_TIME:
+        if user_data.get("glued_time") != TimeConst.DEFAULT_NONE_TIME:
             message_arr.append(
                 "最近被打胶时间: {}".format(
                     ArrowUtil.date_improve(user_data.get("glued_time"))
@@ -359,7 +448,8 @@ class ChinchinInternal:
             )
         # register
         message_arr.append(
-            "注册时间: {}".format(ArrowUtil.date_improve(user_data.get("register_time")))
+            "注册时间: {}".format(ArrowUtil.date_improve(
+                user_data.get("register_time")))
         )
         return message_arr
 
@@ -391,7 +481,8 @@ class Chinchin_me:
                 punish_value = Config.get_lock_me_punish_value()
                 # not need weighting
                 DB.length_decrease(qq, punish_value)
-                message_arr = ["你的牛子还不够长，你🔒不着，牛子自尊心受到了伤害，缩短了{}厘米".format(punish_value)]
+                message_arr = [
+                    "你的牛子还不够长，你🔒不着，牛子自尊心受到了伤害，缩短了{}厘米".format(punish_value)]
                 send_message(qq, group, join(message_arr, "\n"))
             else:
                 message_arr = ["你的牛子太小了，还🔒不到"]
@@ -400,7 +491,8 @@ class Chinchin_me:
             # record record_lock_me_count to qq
             DB.sub_db_badge.record_lock_me_count(qq)
             # FIXME: 因为🔒自己回报高，这样会导致强者一直🔒自己，越强，所以需要一种小概率制裁机制。
-            is_lock_failed = Config.is_hit("lock_me_negative_prob_with_strong_person")
+            is_lock_failed = Config.is_hit(
+                "lock_me_negative_prob_with_strong_person")
             if is_lock_failed:
                 punish_value = Config.get_lock_punish_with_strong_person_value()
                 # not need weighting
@@ -408,7 +500,8 @@ class Chinchin_me:
                 # record record_lock_punish_count to qq
                 DB.sub_db_badge.record_lock_punish_count(qq)
                 # record record_lock_punish_length_total to qq
-                DB.sub_db_badge.record_lock_punish_length_total(qq, punish_value)
+                DB.sub_db_badge.record_lock_punish_length_total(
+                    qq, punish_value)
                 message_arr = ["你的牛子太长了，没🔒住爆炸了，缩短了{}厘米".format(punish_value)]
                 send_message(qq, group, join(message_arr, "\n"))
             else:
@@ -486,14 +579,14 @@ class Chinchin_me:
             "daily_lock_count": 0,
             "daily_pk_count": 0,
             "daily_glue_count": 0,
-            "latest_daily_lock": DEFAULT_NONE_TIME,
-            "latest_daily_pk": DEFAULT_NONE_TIME,
-            "latest_daily_glue": DEFAULT_NONE_TIME,
-            "pk_time": DEFAULT_NONE_TIME,
-            "pked_time": DEFAULT_NONE_TIME,
-            "glueing_time": DEFAULT_NONE_TIME,
-            "glued_time": DEFAULT_NONE_TIME,
-            "locked_time": DEFAULT_NONE_TIME,
+            "latest_daily_lock": TimeConst.DEFAULT_NONE_TIME,
+            "latest_daily_pk": TimeConst.DEFAULT_NONE_TIME,
+            "latest_daily_glue": TimeConst.DEFAULT_NONE_TIME,
+            "pk_time": TimeConst.DEFAULT_NONE_TIME,
+            "pked_time": TimeConst.DEFAULT_NONE_TIME,
+            "glueing_time": TimeConst.DEFAULT_NONE_TIME,
+            "glued_time": TimeConst.DEFAULT_NONE_TIME,
+            "locked_time": TimeConst.DEFAULT_NONE_TIME,
         }
         DB.create_data(new_user)
         message_arr = [
@@ -537,7 +630,10 @@ class Chinchin_with_target:
         target_data = DB.load_data(at_qq)
         user_data = DB.load_data(qq)
         target_length = target_data.get("length")
-        user_length = user_data.get("length")
+        user_length = Chinchin_intercepor.length_weight(
+            origin_length=user_data.get("length"),
+            qq=qq, at_qq=at_qq, source=OpFrom.PK_FROM_LENGTH, 
+        )
         is_user_win = Config.is_pk_win(user_length, target_length)
         DB.record_time(qq, "pk_time")
         DB.record_time(at_qq, "pked_time")
@@ -549,10 +645,10 @@ class Chinchin_with_target:
             else:
                 pk_message = "pk成功了，对面牛子不值一提，你的是最棒的"
             user_plus_value = Chinchin_intercepor.length_operate(
-                qq, Config.get_pk_plus_value(), source=OpFrom.PK_WIN
+                qq, Config.get_pk_plus_value(), source=OpFrom.PK_WIN, at_qq=at_qq
             )
             target_punish_value = Chinchin_intercepor.length_operate(
-                qq, Config.get_pk_punish_value(), source=OpFrom.PK_LOSE
+                qq, Config.get_pk_punish_value(), source=OpFrom.PK_LOSE, at_qq=at_qq
             )
             # weighting from qq
             DB.length_increase(qq, user_plus_value)
@@ -575,7 +671,8 @@ class Chinchin_with_target:
             # record pk_lose_count to qq
             DB.sub_db_badge.record_pk_lose_count(qq)
             # record record_pk_punish_length_total to qq
-            DB.sub_db_badge.record_pk_punish_length_total(qq, user_punish_value)
+            DB.sub_db_badge.record_pk_punish_length_total(
+                qq, user_punish_value)
             message_arr = [
                 "pk失败了，在对面牛子的阴影笼罩下，你的牛子减小了{}厘米，对面牛子增加了{}厘米".format(
                     user_punish_value, target_plus_value
@@ -606,7 +703,8 @@ class Chinchin_with_target:
             send_message(qq, group, join(message_arr, "\n"))
             return
         target_plus_value = Chinchin_intercepor.length_operate(
-            qq, Config.get_lock_plus_value(), source=OpFrom.LOCK_WITH_TARGET
+            qq, Config.get_lock_plus_value(), source=OpFrom.LOCK_WITH_TARGET,
+            at_qq=at_qq
         )
         # weighting from qq
         DB.length_increase(at_qq, target_plus_value)
@@ -649,27 +747,30 @@ class Chinchin_with_target:
         is_glue_failed = Config.is_hit("glue_negative_prob")
         if is_glue_failed:
             target_punish_value = Chinchin_intercepor.length_operate(
-                qq, Config.get_glue_punish_value(), source=OpFrom.GLUE_WITH_TARGET
+                qq, Config.get_glue_punish_value(), source=OpFrom.GLUE_WITH_TARGET_FAIL, at_qq=at_qq
             )
             # weighting from qq
             DB.length_decrease(at_qq, target_punish_value)
             # record record_glue_punish_count to qq
             DB.sub_db_badge.record_glue_punish_count(qq)
             # record record_glue_punish_length_total to qq
-            DB.sub_db_badge.record_glue_punish_length_total(qq, target_punish_value)
+            DB.sub_db_badge.record_glue_punish_length_total(
+                qq, target_punish_value)
             message_arr = ["对方牛子快被大家冲坏了，减小{}厘米".format(target_punish_value)]
             send_message(qq, group, join(message_arr, "\n"))
         else:
             target_plus_value = Chinchin_intercepor.length_operate(
-                qq, Config.get_glue_plus_value(), source=OpFrom.GLUE_WITH_TARGET
+                qq, Config.get_glue_plus_value(), source=OpFrom.GLUE_WITH_TARGET_SUCCESS, at_qq=at_qq
             )
             # weighting from qq
             DB.length_increase(at_qq, target_plus_value)
             # record record_glue_plus_count to qq
             DB.sub_db_badge.record_glue_plus_count(qq)
             # record record_glue_plus_length_total to at_qq
-            DB.sub_db_badge.record_glue_plus_length_total(qq, target_plus_value)
-            message_arr = ["你的打胶让对方牛子感到很舒服，对方牛子增加{}厘米".format(target_plus_value)]
+            DB.sub_db_badge.record_glue_plus_length_total(
+                qq, target_plus_value)
+            message_arr = [
+                "你的打胶让对方牛子感到很舒服，对方牛子增加{}厘米".format(target_plus_value)]
             send_message(qq, group, join(message_arr, "\n"))
 
 
@@ -690,7 +791,8 @@ class Chinchin_upgrade:
             # punish
             punish_length = info["failed_info"]["failed_punish_length"]
             DB.length_decrease(qq, punish_length)
-            message_arr = ["细数牛界之中，贸然渡劫者九牛一生，牛子失去荔枝爆炸了，减小{}厘米".format(punish_length)]
+            message_arr = [
+                "细数牛界之中，贸然渡劫者九牛一生，牛子失去荔枝爆炸了，减小{}厘米".format(punish_length)]
             send_message(qq, group, join(message_arr, "\n"))
             return
         # success
@@ -755,7 +857,7 @@ class Chinchin_farm:
         # 可玩的逻辑, start plant
         plant_info = FarmSystem.start_plant(qq)
         need_time_minutes = plant_info["need_time_minutes"]
-        message_arr = [f"神只会在必要的时候展现他牛子的冰山一角，完成飞升预计需要{need_time_minutes}分钟"]
+        message_arr = [f"神只会在必要的时候展现他牛子的冰山一胶，完成飞升预计需要{need_time_minutes}分钟"]
         send_message(qq, group, join(message_arr, "\n"))
 
     @staticmethod
@@ -780,3 +882,91 @@ class Chinchin_farm:
                     f"牛子修炼结束，你感觉前所未有的舒服，增加了{reward_length}厘米"
                 )
         return is_current_planting
+
+
+class Chinchin_friends:
+    @staticmethod
+    def entry_friends(ctx: dict):
+        qq = ctx["qq"]
+        group = ctx["group"]
+        view = FriendsSystem.get_friends_list_view(qq)
+        message_arr = [view]
+        send_message(qq, group, join(message_arr, "\n"))
+
+    @staticmethod
+    def entry_friends_add(ctx: dict):
+        qq = ctx["qq"]
+        group = ctx["group"]
+        at_qq = ctx["at_qq"]
+        config = FriendsSystem.read_config()
+        max = config["max"]
+        friends_data = FriendsSystem.get_friends_data(qq)
+        # 朋友满了
+        is_friends_limit = len(friends_data["friends_list"]) >= max
+        message_arr = ["不要卷了，你的牛友已经够多了！"]
+        if is_friends_limit:
+            return send_message(qq, group, join(message_arr, "\n"))
+        # 已经是朋友了
+        is_already_friends = at_qq in friends_data["friends_list"]
+        message_arr = ["他已经是你的牛友了，又开始了是吧。"]
+        if is_already_friends:
+            return "\n".join(message_arr)
+        # 准备添加朋友
+        # 计算费用
+        target_friends_data = FriendsSystem.get_friends_data(at_qq)
+        target_shared_count = target_friends_data["friends_share_count"]
+        target_user_length = target_friends_data["length"]
+        daily_need_cost = fixed_two_decimal_digits(
+            config["cost"]["base"] * target_user_length
+            + config["cost"]["share"] * target_shared_count,
+            to_number=True,
+        )
+        current_length = friends_data["length"]
+        is_can_pay_length = current_length >= daily_need_cost
+        if not is_can_pay_length:
+            message_arr = ["自己的牛子都快没了，还想白嫖。"]
+            return send_message(qq, group, join(message_arr, "\n"))
+        # immediate pay
+        DB.length_decrease(qq, daily_need_cost)
+        nickname = target_friends_data.get("latest_speech_nickname")
+        if not nickname:
+            nickname = "无名英雄"
+        message_arr = [
+            f"“这是今天的朋友费...”，“要永远在一起喔o(*￣▽￣*)”，你付出了{daily_need_cost}cm，顺利和{nickname}成为了好朋友！",
+        ]
+        # transfer length
+        will_get_length = daily_need_cost * (1 - config["fee"]["friends"])
+        DB.length_increase(at_qq, will_get_length)
+        # add friend
+        FriendsSystem.add_friends(qq, at_qq)
+        return send_message(qq, group, join(message_arr, "\n"))
+
+    @staticmethod
+    def entry_friends_delete(ctx: dict):
+        # TODO: 友尽需要收费
+        # TODO: 先不支持交友不慎造成的问题，比如交了朋友但是对方退群了，没法 at 他断绝关系了。
+        qq = ctx["qq"]
+        group = ctx["group"]
+        at_qq = ctx["at_qq"]
+        # 检查是否是朋友
+        friends_data = FriendsSystem.get_friends_data(qq)
+        is_already_friends = at_qq in friends_data["friends_list"]
+        if not is_already_friends:
+            message_arr = ["他不是你的牛友，又开始了是吧。"]
+            return send_message(qq, group, join(message_arr, "\n"))
+        # 删除朋友
+        nickname = friends_data.get("latest_speech_nickname")
+        if not nickname:
+            nickname = "无名英雄"
+        message_arr = [
+            f"我要创造一个所有牛子都受伤的世界...，你们都是我的朋友，但也是我的敌人，和{nickname}断绝了关系"]
+        FriendsSystem.delete_friends(qq, at_qq)
+        return send_message(qq, group, join(message_arr, "\n"))
+
+class Chinchin_help():
+
+    @staticmethod
+    def entry_help(ctx: dict):
+        qq = ctx["qq"]
+        group = ctx["group"]
+        send_message(qq, group, HELPPER)
